/*  Copyright (C) 1998-9  Richard Braakman, Anthony Towns
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stddef.h>
#include <stdio.h>
#include <string.h>

#include <unistd.h>
#include <getopt.h>

#include "run-scripts.h"

#define VERSION "1.0"

int verbose = 0;
int debug = 0;
int no_act = 0;

void usage(char *progname) {
    fprintf(stderr, "Usage: %s [OPTION]... [FILE]...\n", progname);
    fprintf(stderr, "Try `%s --help' for more information.\n", progname);
}

void help(char *progname) {
    printf("Usage: %s [OPTION]... [FILE]...\n", progname);
    printf("Run the FILEs in parallel as far as their interdependencies allow.\n"
           "Each FILE must be a script.\n"
           "\n"
           "Options:\n"
           "  -a, --arg=ARG             invoke each FILE with argument ARG\n"
           "  -r, --reverse             reverse needs/provides relationships\n"
           "\n"
           "  -n, --no-act              don't actually run the programs\n"
           "  -v, --verbose             be verbose about what's happening\n"
           "  -d, --debug               include debugging information\n"
           "\n"
           "  -V, --version             print version information and exit\n"
           "  -h, --help                display this help and exit\n");
}

void version(void) {
    printf("run-scripts %s\n", VERSION);
    printf("\nCopyright (C) 1998-9 by Richard Braakman, Anthony Towns\n");
    printf("This is free software; see the source for copying conditions. There is NO\n");
    printf("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");
}

static struct option long_options[] = {
    { "arg", required_argument, NULL, 'a' },
    { "reverse", no_argument, NULL, 'r' },
    { "no-act", no_argument, NULL, 'n' },
    { "verbose", no_argument, NULL, 'v' },
    { "debug", no_argument, NULL, 'd' },
    { "version", no_argument, NULL, 'V' },
    { "help", no_argument, NULL, 'h' },
    { 0, 0, 0, 0 }
};

int main(int argc, char *argv[]) {
    int c;
    char *scriptarg = NULL;
    int reverse = 0;

    for (;;) {
	c = getopt_long(argc, argv, "a:rnvdhV", long_options, NULL);
	if (c == EOF)
	    break;

	switch (c) {
	case 'a':
	    scriptarg = strdup(optarg);
	    break;

	case 'r':
	    reverse = 1;
	    break;

	case 'v':
	    verbose = 1;
	    break;

	case 'd':
	    debug = 1;
	    break;

	case 'n':
	    no_act = 1;
	    break;

	case 'V':
	    version();
	    exit(0);

	case 'h':
	    help(argv[0]);
	    exit(0);

	default:
	    usage(argv[0]);
	    exit(1);
	}
    }

    if (argc - optind == 0) {
	usage(argv[0]);
	exit(1);
    }

    if (run_scripts(argv + optind, argc - optind, scriptarg, reverse) < 0) {
	exit(1);
    }
    exit(0);
}
